@@ -1,6 +1,6 @@
 import json
 from six import iteritems
-from traitlets import HasTraits, Unicode, List, Bool, Int, Dict, Any, Bytes, validate
+from traitlets import HasTraits, Unicode, List, Bool, Int, Dict, Any, Bytes, Union, validate
 from .data import type_detect
 from .validate import validate_view, validate_columns, validate_rowpivots, validate_columnpivots, validate_aggregates, validate_sort, validate_computedcolumns, validate_filters, validate_plugin_config
 from .schema import validate_schema
@@ -9,7 +9,7 @@ from .schema import validate_schema
 class PerspectiveBaseMixin(HasTraits):
     '''Perspective Base Mixin'''
     # Data (private)
-    _data = List(default_value=[]).tag(sync=True)
+    _data = Union((List(default_value=[]), Dict())).tag(sync=True)
     _bin_data = Bytes().tag(sync=True)  # binary data
     _dat_orig = Any()
 
@@ -39,8 +39,11 @@ class PerspectiveBaseMixin(HasTraits):
     # dark mode
     dark = Bool(False).tag(sync=True)
 
+    # try to use apache arrow to transfer data
+    transfer_as_arrow = Bool(True).tag(sync=True)
+
     def load(self, value):
-        data_object = type_detect(value, schema=self.schema)
+        data_object = type_detect(value, schema=self.schema, columns=self.columns, transfer_as_arrow=self.transfer_as_arrow)
         self.datasrc = data_object.type
         if data_object.type in ('arrow'):
             self._data = []
@@ -59,7 +62,7 @@ class PerspectiveBaseMixin(HasTraits):
                         computedcolumns.append(c['name'])
 
             if not self.columns and 'columns' not in data_object.kwargs:
-                columns = list(map(lambda x: str(x), s.keys()))
+                columns = list(map(lambda x: str(x), data_object.columns))
 
                 # reasonable default, pivot by default in non-grid view
                 if not self.rowpivots and self.view != 'hypergrid':
@@ -163,6 +166,7 @@ class PerspectiveBaseMixin(HasTraits):
               settings=True,
               embed=False,
               dark=False,
+              transfer_as_arrow=False,
               *args,
               **kwargs):
         '''Setup perspective base class
@@ -200,6 +204,7 @@ class PerspectiveBaseMixin(HasTraits):
                 use dark theme
 
         '''
+        self.transfer_as_arrow = transfer_as_arrow
         self.view = validate_view(view)
         self.schema = schema or {}
         self.sort = validate_sort(sort) or []
